extends Node2D

@onready var tile_map: TileMap = $"../TileMap"

@export var range: int

enum STATES {
	WAIT,
	IN_MOVE
}

var _possible_moves: Array
var _state: STATES = STATES.WAIT

func _physics_process(delta):
	match _state:
		STATES.IN_MOVE:
			if Input.is_mouse_button_pressed(MOUSE_BUTTON_MIDDLE):
				print("self: %s, state: %s" % [self, _state])
			if Input.is_mouse_button_pressed(MOUSE_BUTTON_LEFT):
				print("mooooove")
				move()

func set_state(new_state: STATES) -> void:
	if new_state == _state:
		return
	_state = new_state

func move():
	#convert the current Mouse Position to a Tilemap Position
	var cur_map_position = tile_map.get_tilemap_position(global_position)
	var mouse_global_pos = get_global_mouse_position()
	var mouse_map_pos = tile_map.get_tilemap_position(mouse_global_pos)
	var mouse_local_pos = tile_map.get_local_position(mouse_map_pos)
	#the possible moves are created beforehand with the _set_possible_moves Function
	for pos_mov in _possible_moves:
		if pos_mov == mouse_map_pos:
			if tile_map.test_if_moveable_to_tile(mouse_map_pos):
				print("move to: ", mouse_local_pos)
				global_position = mouse_local_pos
				_erase_possible_moves(_possible_moves)
				set_state(STATES.WAIT)
			else:
				print("dont move to: ", mouse_local_pos)
				#display a Image, that this move is illegal and remove the image after 2 seconds
				#uses the Tilemap with the ID 2 and selects a certain Part from it
				tile_map.set_cell(3, pos_mov, 2, Vector2i(12, 10), 0)
				var wrong_tile_timer = Timer.new()
				wrong_tile_timer.wait_time = 2.0
				wrong_tile_timer.one_shot = true
				wrong_tile_timer.autostart = true
				add_child(wrong_tile_timer)
				wrong_tile_timer.start()
				wrong_tile_timer.timeout.connect(_on_wrong_tile_timer_timeout.bind(pos_mov))
				_erase_possible_moves(_possible_moves)
		else:
			_erase_possible_moves(_possible_moves)

func _on_wrong_tile_timer_timeout(coords: Vector2i):
	print("timeout")
	tile_map.erase_cell(3, coords)

func _erase_possible_moves(coords: Array) -> void:
	for coord in coords:
		tile_map.erase_cell(2, coord) #2 = hover layer id from tilemap
	_possible_moves = [] #reset the possible moves, so no movement is able
	set_state(STATES.WAIT)

func _set_possible_moves(cur_pos: Vector2i) -> void:
	var x = cur_pos.x
	var y = cur_pos.y
	var positions = []
	#range is exported variable, which corresponds to a stat of this character
	#in this example, I disallowed diagonal movement
	#if you want to allow diagonal movement as well, you have to change this part
	for i in range(-range, range + 1):
		for j in range(-range, range + 1):
			if abs(i) + abs(j) <= range:
				var new = Vector2i(x+i, y+j)
				if new in tile_map.grid_floor:
					positions.append(new)
	
	positions.erase(cur_pos) #so there is no green in our current positions, looks annoying
	for i in positions:
		if tile_map.test_if_moveable_to_tile(i):
			tile_map.set_cell(2, i, 3, Vector2i(0, 0), 0)
			#2 = hover layer id from tilemap
			#i = coordinates
			#3 = id of the 16x16 Image filled with green
			#Vector2i(0, 0) = coords of the image
	_possible_moves = positions


func _on_move_button_pressed():
	set_state(STATES.IN_MOVE)
	var cur_map_position = tile_map.get_tilemap_position(global_position)
	_set_possible_moves(cur_map_position)
