extends TileMap

var grid_floor
var grid_hover
var grid_obstacles
var tile_size

func _ready():
	initate_grids()
	#add_coord_labels_to_each_tile() #just for debugging, comment out if not needed

func _process(delta):
	if Input.is_mouse_button_pressed(MOUSE_BUTTON_RIGHT):
		print_positions()

func initate_grids() -> void:
	#adjust the layer ids to your project
	#to get the layer id, hover above the corresponding Layer in the TileMap Inspector
	grid_floor = get_used_cells(0)
	grid_obstacles = get_used_cells(1)
	grid_hover = get_used_cells(2)
	tile_size = get_quadrant_size()
	print("grid_floor: ", grid_floor)
	print("grid_obstacles: ", grid_obstacles)
	print("grid_hover: ", grid_hover)

func get_tilemap_position(pos: Vector2i) -> Vector2i:
	return local_to_map(pos)

func get_local_position(pos: Vector2i) -> Vector2i:
	#gets the center position of the tile
	return map_to_local(pos)

func print_positions() -> void:
	var global_pos = get_global_mouse_position()
	var tilemap_pos = get_tilemap_position(global_pos)
	print("Global Position: ", global_pos)
	print("Map Position: ", tilemap_pos)


func add_coord_labels_to_each_tile():
	#grid_floor is initiated in initate_grids()
	#add a label to each tile in the runtime
	for tile_coord in grid_floor:
		var world_pos = map_to_local(tile_coord)
		var label = Label.new()
		label.text = str(tile_coord.x) + "," + str(tile_coord.y)
		print("tile_coord local: ", tile_coord)
		print("tile_coord global: ", world_pos)
		label.position = Vector2(world_pos.x-7,world_pos.y)
		label.add_theme_font_override("font", load("res://4x6px.ttf"))
		label.add_theme_font_size_override("font_size", 16)
		label.z_index = 1
		add_child(label)

func test_if_moveable_to_tile(tile_coords: Vector2i) -> bool:
	const layers_to_test: Array = [0,1,2,3] #which layers should be excluded from the Tiles where the player can move to
	const custom_data_layer_names: Array = ["water", "tree"]
	var result = true
	for layer in layers_to_test:
		var data = get_cell_tile_data(layer, tile_coords)
		if data != null:
			var custom_data: Array
			for custom_layer_name in custom_data_layer_names:
				custom_data.append({custom_layer_name: data.get_custom_data(custom_layer_name)})
			var result_data = {
				"coords": tile_coords,
				"layer": layer,
				"data": custom_data
			}
			for i in custom_data:
				if "water" in i:
					if i["water"] == true:
						result = false
				elif "tree" in i:
					if i["tree"] == true:
						result = false
	return result
